
from .errors import *
from .models import *

from .chapterdownload import download_chapter

from .storypage import (
	get_story_page,
	StoryPage
)

from .storysearch import (
	get_search_results,
	Chapter,
	SearchResult
)

from .hometooltips import get_home_page


def get_chapter_list(url: str) -> list[Chapter]:
	return get_story_page(url).chapter_list()



import requests
from manganelo.common.constants import HOME_TOOLTIPS_URL

from manganelo.models import HomeStoryTooltip
from manganelo.errors import RequestError


def get_home_page() -> list[HomeStoryTooltip]:
    r = requests.get(HOME_TOOLTIPS_URL)

    if r.status_code != 200:
        raise RequestError(f"Home page request failed with status code {r.status_code}")

    try:
        return [HomeStoryTooltip(ele) for ele in r.json()]
    except KeyError as e:
        raise RequestError(f"Home page response differs from expected response") from e
    except Exception as e:
        raise RequestError(f"An unexpected error occurred") from e



import ast
import string

import functools as ft

from bs4 import BeautifulSoup

from manganelo import siterequests
from manganelo.common import utils

from manganelo.storypage import get_story_page, Chapter


class SearchResult:
	def __init__(self, soup):
		self._soup = soup

	@ft.cached_property
	def title(self) -> str: return self._soup.find(class_="item-img").get("title")

	@ft.cached_property
	def url(self) -> str: return self._soup.find(class_="item-img").get("href")

	@ft.cached_property
	def icon_url(self) -> str: return self._soup.find("img", class_="img-loading").get("src")

	@ft.cached_property
	def updated(self):
		s = self._soup.find("span", class_="text-nowrap item-time").text

		return utils.parse_date(s, "Updated : %b %d,%Y - %H:%M")

	@ft.cached_property
	def authors(self) -> list[str]:
		return [e.strip() for e in self._soup.find("span", class_="text-nowrap item-author").text.split(",")]

	@ft.cached_property
	def views(self):
		s = self._soup.find_all("span", class_="text-nowrap item-time")[-1].text

		return ast.literal_eval(s.replace("View : ", "").replace(",", ""))

	@ft.cached_property
	def rating(self) -> float: return float(self._soup.find("em", class_="item-rate").text)

	@ft.cache
	def chapter_list(self) -> list[Chapter]:
		return get_story_page(self.url).chapter_list()

	def download_icon(self, path: str):
		if img := siterequests.get_image(self.icon_url):
			return utils.save_image(img, path)


def get_search_results(title: str) -> list[SearchResult]:
	allowed_characters: str = string.ascii_letters + string.digits + "_"

	title = "".join([char.lower() for char in title.strip().replace(" ", "_") if char in allowed_characters])

	r = siterequests.search(title)

	soup = BeautifulSoup(r.content, "html.parser")

	return [SearchResult(ele) for ele in soup.find_all(class_="search-story-item")]



import requests
import urllib.parse

from manganelo.common.constants import ROOT_URL


def get(url):
    return requests.get(url, stream=True, timeout=10)


def search(title):
    return get(f"{ROOT_URL}search/story/{title}")


def get_image(url):
    header = {
        'Accept': 'image/png,image/svg+xml,image/*;q=0.8,video/*;q=0.8,*/*;q=0.5',
        'Accept-Encoding': 'gzip, deflate, br',
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/605.1.15 (KHTML, like Gecko) '
                      'Version/13.1.2 Safari/605.1.15',
        'Host': urllib.parse.urlparse(url).netloc, 'Accept-Language': 'en-ca', 'Referer': ROOT_URL,
        'Connection': 'keep-alive'
    }

    return requests.get(url, stream=True, timeout=10, headers=header)



import re
import ast

import functools as ft

from bs4 import BeautifulSoup

from . import siterequests
from .common import utils

from .errors import RequestError
from .chapterdownload import download_chapter


class Chapter:
	def __init__(self, soup):
		self._soup = soup

	@ft.cached_property
	def title(self): return self._soup.find("a").text

	@ft.cached_property
	def url(self): return self._soup.find("a").get("href")

	@ft.cached_property
	def chapter(self):
		return ast.literal_eval(re.split("[-_]", self.url.split("chapter")[-1])[-1])

	@ft.cached_property
	def views(self):
		s = self._soup.find_all("span", class_="chapter-view text-nowrap")[-1].text
		return ast.literal_eval(s.replace(",", ""))

	@ft.cached_property
	def uploaded(self):
		s = self._soup.find("span", class_="chapter-time text-nowrap").get("title")

		return utils.parse_date(s, "%b %d,%Y %H:%M")

	def download(self, path):
		return download_chapter(self.url, path)


class StoryPage:
	def __init__(self, url, soup):
		self._soup = soup

		self.url = url

	@ft.cached_property
	def title(self): return self._soup.find(class_="story-info-right").find("h1").text.strip()

	@ft.cached_property
	def authors(self):
		values = self._soup.find("table", class_="variations-tableInfo").find_all("td", class_="table-value")
		author = values[1]

		return [e.strip() for e in author.text.split(",")]

	@ft.cached_property
	def genres(self):
		values = self._soup.find("table", class_="variations-tableInfo").find_all("td", class_="table-value")
		genres = values[3].find_all("a", class_="a-h")

		return [e.text.strip() for e in genres]

	@ft.cached_property
	def updated(self):
		values = self._soup.find("div", class_="story-info-right-extent").find_all("span", class_="stre-value")

		return utils.parse_date(values[0].text.strip(), "%b %d,%Y - %H:%M %p")

	@ft.cached_property
	def views(self):
		values = self._soup.find("div", class_="story-info-right-extent").find_all("span", class_="stre-value")

		s = values[1].text.strip()

		return ast.literal_eval(s.replace(",", ""))

	@ft.cached_property
	def icon_url(self): return self._soup.find("div", class_="story-info-left").find("img", class_="img-loading")["src"]

	@ft.cached_property
	def description(self): return self._soup.find("div", class_="panel-story-info-description").text.strip()

	@ft.cache
	def chapter_list(self):
		panels = self._soup.find(class_="panel-story-chapter-list")

		return [Chapter(ele) for ele in panels.find_all(class_="a-h")[::-1] if ele is not None]

	def download_icon(self, *, path: str):
		if img := siterequests.get_image(self.icon_url):
			return utils.save_image(img, path)


def get_story_page(url):
	r = siterequests.get(url)

	soup = BeautifulSoup(r.content, "html.parser")

	if "404" in soup.find("title").text:
		raise RequestError(f"Page '{url}' was not found")

	return StoryPage(url, soup)
